@@ -81,17 +81,23 @@ use Storable 'freeze';
   $record_class->cache_cache() : $cache_cache
   $record_class->cache_cache( $cache_cache ) 
 
+Gets or sets the cache object associated with this record class.
+
+If no cache has been set for a given class, no caching is performed.
+
 =back
 
 B<Cache Object Requirements:> This package in intended to work with cache object that use the Cache::Cache interface. However, any package which support the limited cache interface used by this package should be sufficient. 
 
-=over 4
+Two small classes are included that support this interface; see L<DBIx::SQLEngine::Cache::TrivialCache> and  L<DBIx::SQLEngine::Cache::BasicCache>.
 
-=item new()
+The following methods are used
+
+=over 4
 
 Constructor.
 
-=item namespace()
+=item get_namespace()
 
 Used to differentiate one cache object from another.
 
@@ -109,8 +115,6 @@ Clear some or all values in the cache.
 
 =back
 
-B<Included Cache Classes:> Two small classes are included that support this interface; see L<DBIx::SQLEngine::Cache::TrivialCache> and  L<DBIx::SQLEngine::Cache::BasicCache>.
-
 =cut
 
 use Class::MakeMethods (
@@ -123,28 +127,57 @@ use Class::MakeMethods (
 
 =over 4
 
+=item cache_key()
+
+  $record_class->cache_key( $key ) : $string_value
+  $record_class->cache_key( \@key ) : $string_value
+  $record_class->cache_key( \%key ) : $string_value
+
+Returns the string value to be used as a cache key. The argument may be an existing string, a reference to a shallow array whose elements will be joined with "\0/\0", or any other reference value which will be stringified by Storable.
+
 =item cache_get()
 
   $record_class->cache_get( $key ) : $value
   $record_class->cache_get( $key ) : ( $value, $updater_code_ref )
 
+Returns the cached value associated with this key, if any. If called in a list context, also returns a reference to a subroutine which will save a new value for that key.
+
 =item cache_set()
 
   $record_class->cache_set( $key, $value )
 
+Caches this value under the provided key.
+
 =item cache_get_set()
 
   $record_class->cache_get_set( $key, $code_ref, @args ) : $value
 
+Returns the curent value provided by a cache_get on the provided key, or if it is undefined, invokes the subroutine reference with any additional arguments provided, and saves the subroutine's return value as the cached value.
+
 =item cache_clear()
 
   $record_class->cache_clear()
   $record_class->cache_clear( $key )
 
+Clear all values from the cache, or just those associated with the given key.
+
 =back
 
 =cut
 
+sub cache_key {
+  my ( $self, $key ) = @_;
+  my $type = ref($key);
+  if ( ! $type ) { 
+    $key 
+  } elsif ( $type eq 'ARRAY' ) {
+    join("\0/\0", @$key) 
+  } else { 
+    local $Storable::canonical = 1; 
+    freeze($key) 
+  }
+}
+
 # $value = $self->cache_get( $key );
 # ( $value, $update ) = $self->cache_get( $key );
 sub cache_get {
@@ -152,7 +185,7 @@ sub cache_get {
   
   my $cache = $self->cache_cache() or return;
   
-  $key = do { local $Storable::canonical = 1; freeze($key) } if ( ref $key );
+  $key = $self->cache_key($key) if ( ref $key );
   my $current = $cache->get( $key );
   
   if ( ! defined $current ) {
@@ -173,7 +206,7 @@ sub cache_set {
   
   my $cache = $self->cache_cache() or return;
   
-  $key = do { local $Storable::canonical = 1; freeze($key) } if ( ref $key );
+  $key = $self->cache_key($key) if ( ref $key );
   
   $self->cache_log_operation( $cache, 'write', $key );
   $cache->set( $key, @value );
@@ -217,14 +250,27 @@ sub cache_clear {
 
 =item CacheLogging()
 
+  $record_class->CacheLogging() : $level
+  $record_class->CacheLogging( $level )
+
+Sets the logging level associated with a given class. 
+
 =item cache_log_operation()
 
+  $record_class->cache_log_operation( $cache, $operation, $key ) 
+
+Does nothing unless a CacheLogging level is set for this class.
+
+Uses warn() to print a message to the error log, including the key string used, and the operation, which will be one of "hit", "miss", "write", and "clear".
+
+If the level is greater than one, the message will also include a history of prior operations on this key.
+
 =back
 
 =cut
 
 use Class::MakeMethods (
-  'Standard::Inheritable:scalar' => 'CacheLogging',
+  'Template::ClassInherit:scalar' => 'CacheLogging',
 );
 
 use vars qw( %CachingHistory );
@@ -233,14 +279,12 @@ sub cache_log_operation {
   my ( $self, $cache, $oper, $key ) = @_;
   my $level = $self->CacheLogging() or return;
   my $namespace = $cache->get_namespace;
-  if ( $level < 2 ) {
-    warn "Cache $namespace: $oper " . DBIx::SQLEngine::printable($key) . "\n";
-  } else {
+  if ( $level > 1 ) {
     my $history = ( $CachingHistory{ $key } ||= [] );
-    warn "Cache $namespace: $oper (" . join(' ', @$history ) . ") "  .
-				      DBIx::SQLEngine::printable($key)."\n";
+    $oper .= " (" . join(' ', @$history ) . ")";
     push @$history, $oper;
   }
+  warn "Cache $namespace: $oper " . DBIx::SQLEngine::printable($key) . "\n";
 } 
 
 ########################################################################
@@ -254,7 +298,7 @@ sub cache_log_operation {
   DBIx::SQLEngine->define_cache_styles( $name, $code_ref )
   DBIx::SQLEngine->define_cache_styles( %names_and_code_refs )
 
-Define a named caching style.
+Define a named caching style. The code ref supplied for each name should create and return an object from the Cache::Cache hierarchy, or another caching class which supports the interface described in the "Cache Object Requirements" section above.
 
 =item cache_styles()
 
@@ -269,6 +313,11 @@ Accessor for global hash mapping cache names to initialization subroutines.
 =item use_cache_style()
 
   $class_name->use_cache_style( $cache_style_name )
+  $class_name->use_cache_style( $cache_style_name, @options )
+
+Uses the named caching definition to create a new cache object, and associates it with the given class.
+
+Use one of the predefined caching styles described in the "Default Caching Styles" section below, or define your own cache styles with define_cache_styles.
 
 =back
 
@@ -294,7 +343,7 @@ sub use_cache_style {
 
 =pod
 
-B<Defaults:> The following cache styles are predefined. Except for 'simple', using any of these styles will require installation of the Cache::Cache distribution.
+B<Default Caching Styles:> The following cache styles are predefined. Except for 'simple', using any of these styles will require installation of the Cache::Cache distribution.
 
 =over 4
 
@@ -320,6 +369,22 @@ Uses Cache::FileCache with a default expiration time of 30 seconds.
 
 =back
 
+B<Examples:>
+
+=over 2
+
+=item *
+
+  # requires DBIx::SQLEngine::Cache::TrivialCache
+  $class_name->use_cache_style('simple');
+
+=item *
+
+  # requires Cache::FastMemoryCache from CPAN
+  $class_name->use_cache_style('active'); 
+
+=back
+
 =cut
 
 __PACKAGE__->define_cache_styles( 
@@ -371,17 +436,22 @@ __PACKAGE__->define_cache_styles(
 
 =head1 FETCHING DATA (SQL DQL)
 
+Each of these methods provides a cached version of the superclass method. 
+The results of queries are cached based on the SQL statement and parameters used. 
+
 =over 4
 
 =item fetch_select()
 
-  $class_name->fetch_select ( %select_clauses ) : $record_set
+  $class_name->fetch_select( %select_clauses ) : $record_set
 
+Retrives records from the table using the provided SQL select clauses. 
 
 =item fetch_one_record()
 
   $sqldb->fetch_one_record( %select_clauses ) : $record_hash
 
+Retrives one record from the table using the provided SQL select clauses. 
 
 =item select_record()
 
@@ -389,34 +459,51 @@ __PACKAGE__->define_cache_styles(
   $class_name->select_record ( \@compound_primary_key ) : $record_obj
   $class_name->select_record ( \%hash_with_primary_key_value ) : $record_obj
 
+Fetches a single record by primary key.
 
 =item select_records()
 
   $class_name->select_records ( @primary_key_values_or_hashrefs ) : $record_set
 
+Fetches a set of one or more records by primary key.
 
 =item visit_select()
 
   $class_name->visit_select ( $sub_ref, %select_clauses ) : @results
   $class_name->visit_select ( %select_clauses, $sub_ref ) : @results
 
+Calls the provided subroutine on each matching record as it is retrieved. Returns the accumulated results of each subroutine call (in list context).
+
+To Do: This could perform caching of the matched records, but currently does not.
+
+=back
+
+The conversion of select clauses to a SQL statement is performed by the sql_select method:
+
+=over 4
+
+=item sql_select()
+
+  $class_name->sql_select ( %sql_clauses ) : $sql_stmt, @params
+
+Uses the table to call the sql_select method on the current SQLEngine driver. 
 
 =back
 
 =cut
 
-
-
 # $records = $record_class->fetch_select( %select_clauses );
 BEGIN { push @MIXIN, "#line ".__LINE__.' "'.__FILE__.'"', "", <<'/' }
 sub fetch_select {
   my $self = shift;
   my %clauses = @_;
   
-  my ($records, $update) = $self->cache_get( $self->cache_key_for_fetch( %clauses ) );
+  my @sql = $self->sql_select( %clauses );
+
+  my ($records, $update) = $self->cache_get( \@sql );
   
   if ( ! defined $records ) {
-    $records = $self->SUPER::fetch_select( %clauses );
+    $records = $self->SUPER::fetch_select( sql => \@sql );
     $update->( $records ) if ( $update and $records );
   }
   
@@ -435,12 +522,34 @@ sub select_record {
   $self->fetch_one_record( where => $self->get_table()->primary_criteria($id) )
 }
 
+# $rows = $self->select_rows( @ids_or_hashes );
+sub select_records {
+  my ( $self, @ids ) = @_;
+  $self->fetch_select( where => $self->get_table()->primary_criteria(@ids) )
+}
+
+# @results = $self->visit_select( %select_clauses, $sub );
+BEGIN { push @MIXIN, "#line ".__LINE__.' "'.__FILE__.'"', "", <<'/' }
+sub visit_select {
+  my $self = shift;
+  my $sub = ( ref($_[0]) ? shift : pop );
+  my %clauses = @_;
+  
+  my @sql = $self->sql_select( %clauses );
+  
+  my ($records, $update) = $self->cache_get( \@sql );
+  
+  if ( $records ) {
+    return map &$sub( $_ ), @$records;
+  } 
+  $self->sqlengine_do('visit_select', @_, $sub )
+}
+/
+
 ########################################################################
 
-sub cache_key_for_fetch {
-  my ($self, %clauses) = @_;
-  
-  join "\0/\0", $self->get_table->sqlengine_do( 'sql_select', %clauses )
+sub sql_select {
+  (shift)->get_table->sqlengine_do( 'sql_select', @_ );
 }
 
 ########################################################################
